#include <getopt.h>
#include "affy.h"

int write_probeset(AFFY_CHIP *cp, AFFY_PROBESET *ps);

/* Administrative options */
char *progname;
char version[]="$Id: main.c 112 2005-03-17 19:56:33Z eschris $";

char *options="hf:t:";
struct option longopts[]= {
  {"help",no_argument,0,'h'},
  {"file",required_argument,0,'f'},
  {"type",required_argument,0,'t'},
  {0,0,0,0}
};
char *usage=\
"%s [ --help | -h ] [file.CEL file.CEL]\n"\
" -f file  Use file as the dat/cel file.\n"\
" -t type  What file type as output (tiff|text)\n"\
"";


#define OT_TEXT 1
#define OT_TIFF 2
char *ext="txt";
int   OUTPUT_TYPE=OT_TEXT;

int main(int argc, char **argv) 
{
  int o,option_index;
  char *file=NULL;
	char **probeset_list;
	int i,j;
	AFFY_CHIP *cp;

	
  progname=argv[0];
  while ((o=getopt_long(argc,argv,options,longopts,&option_index)) != EOF){
    switch(o) {
		case 't':
			if ( !strcmp(optarg,"tif") || !strcmp(optarg,"tiff") ) {
				OUTPUT_TYPE=OT_TIFF;
				ext="tiff";
			} else {
				OUTPUT_TYPE=OT_TEXT;
				ext="txt";
			}
			break;
    case 'f':
    	file=strdup(optarg);
    	break;
    case 'h':
      fprintf(stderr,usage,progname);
      exit(1);
    }
  }

  if ( argv[optind]==NULL ) {
		fprintf(stderr,"You must supply probesets to extract\n");
		fprintf(stderr,usage,progname);
		exit(-1);
  }
	/* All other arguments are probesets */
	probeset_list=argv+optind;
	
	/* Try to load data. Errors should result in dying. */
	cp=calloc(1,sizeof(AFFY_CHIP));
	cp->dat=affy_load_dat_file(file);
	cp->cdf=affy_load_cdf_file(cp->dat->probe_array_type,NULL);
	
	/* Identify a probeset */
	for (i=0; probeset_list[i] != NULL; i++) {
		/* Try and find the probeset in the chip */
		for (j=0; j < cp->cdf->numprobesets; j++) {
			if ( !strcmp(probeset_list[i],cp->cdf->probeset[j].name))
				write_probeset(cp,&(cp->cdf->probeset[j]));
		}
	}
	return 0;
}

/*
 * Given a probeset, write out data for each probe,
 * for mm and pm.
 */
int write_probeset(AFFY_CHIP *cp, AFFY_PROBESET *ps)
{
	int i;
	AFFY_PIXREGION *pr_pm, *pr_mm;
	char buf_pm[1024], buf_mm[1024];
	
	for (i=0; i < ps->numprobes; i++) {
		AFFY_PROBE *p=&(ps->probe[i]);
		/* PM */	
		pr_pm=affy_pixels_from_cell(cp, p->pm.x,p->pm.y);
		sprintf(buf_pm,"%s-pm-%d.%s",ps->name,i,ext);
		pr_mm=affy_pixels_from_cell(cp,p->mm.x,p->mm.y);
		sprintf(buf_mm,"%s-mm-%d.%s",ps->name,i,ext);

		switch (OUTPUT_TYPE) {
			case OT_TEXT:
				affy_pixregion2text(pr_pm,buf_pm);
				affy_pixregion2text(pr_mm,buf_mm);
				break;
			case OT_TIFF:
				affy_pixregion2tiff(pr_pm,buf_pm);
				affy_pixregion2tiff(pr_mm,buf_mm);
				break;
		}
	}		
	return 0;
}