#include <stdio.h>
#include <getopt.h>
#include <sys/types.h>
#include <dirent.h>
#include "utils.h"
#include "affy.h"
#include "affy_mas5.h"

char *output_file="exprs-mas.txt";
MAS5_FLAGS flags;

/* Administrative options */
char *progname;
char version[]="$Id: main.c 200 2005-11-11 22:03:37Z eschrich $";

char *options="hd:o:";
struct option longopts[]= {
  {"mean-normalize",no_argument,&(flags.use_mean_normalization),1},
  {"background",no_argument,&(flags.use_background_correction),1},
  {"no-background",no_argument,&(flags.use_background_correction),0},
	{"directory",required_argument,0,'d'},
  {"help",no_argument,0,'h'},
  {"output",required_argument,0,'o'},
  {"scale",no_argument,&(flags.use_probeset_scaling),1},
  {"no-scale",no_argument,&(flags.use_probeset_scaling),0},
  {"cdf",required_argument,0,'c'},
  {0,0,0,0}
};
char *usage=\
"%s [ --help | -h ] [-b|-B] [-n|-N] [ -t type ]file.CEL file.CEL\n"\
"   --background           use background correction\n"\
"      --no-background\n"\
"   --mean-normalize       normalize (by mean)\n"\
"   --scale                Scale probesets to a constant factor\n"\
"      --no-scale\n"\
"   -d dir                 Load all cel files from dir\n"\
"   -o output              Store output file\n"\
"   --cdf=dir              Directory for cdf file\n"\
"";

int main(int argc, char **argv) 
{
	int o,option_index;
	char *directory=".";
	char **filelist;
  AFFY_CHIPSET *c;
 
  
	
	progname=argv[0];
	mas5_set_defaults(&flags);
	while ((o=getopt_long(argc,argv,options,longopts,&option_index)) != EOF){
	switch(o) {
		case 'd':
			directory=strdup(optarg);
			break;
		case 'o':
			output_file=strdup(optarg);
			break;
		case 'c':
			flags.cdf_directory=strdup(optarg);
			break;
    case 'h':
      fprintf(stderr,usage,progname);
      exit(1);
    }
  }

	
		
	 /* If files is NULL, open all CEL files in the current directory */
  if ( argv[optind]==NULL ) {
		filelist=affy_list_files(directory,".cel");
  } else {
    /* Allocate storage for as many chips as we expect */
    filelist=argv+optind;
  }
  
  /* More efficient approach: do them one at a time */
	c=mas5(filelist,&flags);
	affy_write_expressions(c,output_file);
	
	return 0;
}

  
