#include <stdio.h>
#include <getopt.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>

#include "utils.h"
#include "affy.h"
#include "affy_rma.h"


char *output_file="exprs-rma.txt";
RMA_FLAGS flags;



/* Administrative options */
char *progname;
char version[]="$Id: main.c 201 2005-11-11 22:03:48Z eschrich $";

char *options="hm::d:s:c:o:";
struct option longopts[]= {
  {"normalize",no_argument,&(flags.use_normalization),1},
  {"no-normalize",no_argument,&(flags.use_normalization),0},
  {"background",no_argument,&(flags.use_background_correction),1},
  {"no-background",no_argument,&(flags.use_background_correction),0},
  {"mean-normalize",optional_argument,0,'m'},
  {"dir",required_argument,0,'d'},
  {"save-intermediate",required_argument,0,'s'},
  {"combine",required_argument,0,0},
  {"probe-list",required_argument,0,0},
  {"help",no_argument,0,'h'},
  {"cdf",required_argument,0,'c'},
  {"output",required_argument,0,'o'},
  {0,0,0,0}
};
char *usage=\
"%s [ --help | -h ] [file.CEL file.CEL]\n"\
" --normalize  Normalize data\n"\
"   --no-normalize\n"\
" --background  Background correct data \n"\
"   --no-background\n"\
" -d dir  Use dir as the working directory\n"\
"   --dir=dir\n"\
" -s file  save intermediate results to file and free memory\n"\
"   --save-intermediate=file\n"\
" --combine=filelist  filelist of intermediate images \n"\
" --probe-list=filelist  filelist of probesets\n"\
" -c cdfdir Use cdfdir to find the cdf file\n"\
"   --cdf=cdfdir\n"\
" -o f      Save exprs to f\n"\
"   --output=f\n"\
" -m m     Mean normalize to target m\n"\
"   --mean-normalize m\n"\
"";



int main(int argc, char **argv) 
{
  AFFY_CHIPSET *c;
  int o,option_index;
  char *directory=".";
  char **filelist;
	
	
  progname=argv[0];
  rma_set_defaults(&flags);
  while ((o=getopt_long(argc,argv,options,longopts,&option_index)) != EOF){
    switch(o) {
    case 0:
      if (strcmp(longopts[option_index].name, "combine") == 0) {
	flags.list_intermediate=strdup(optarg);
      }
      if (strcmp(longopts[option_index].name, "probe-list") == 0) {
	flags.probe_list=strdup(optarg);
      }
      break;
    case 'd':
      directory=strdup(optarg);
      break;
    case 's':
      flags.intermediate_file=strdup(optarg);
      break;
    case 'M':
      flags.use_mean_normalization=True;
      if ( optarg != NULL ) {
      	flags.mean_normalization_target_value=atof(optarg);
      }
      break;
    case 'c':
    	flags.cdf_directory=strdup(optarg);
    	break;
    case 'o':
    	output_file=strdup(optarg);
    	break;
    case 'h':
      fprintf(stderr,usage,progname);
      exit(1);
    }
  }
  
  flags.output_filename = output_file;
  
  if (flags.list_intermediate != NULL) {
    c=combine_intermediate(flags.list_intermediate, &flags);
    /* Finally write out the entire expression array */
    if (c != NULL) 
      affy_write_expressions(c,output_file);
    exit(0);
  }
  
  /* If files is NULL, open all CEL files in the current directory */
  if ( argv[optind]==NULL ) {
    char **zlist;
    char **list;
    int zcount = 0;
    int lcount = 0;
    char **p;
    list=affy_list_files(directory,".cel");
    zlist=affy_list_files(directory,".cel.gz");
    /* Count up chips */
    for (p = list, lcount = 0; *p != NULL; p++)
      lcount++;
    for (p = zlist, zcount = 0; *p != NULL; p++)
      zcount++;
    filelist = REALLOC(zlist, (zcount + lcount + 1) * sizeof(char *));
    for (p = list, lcount = 0; *p != NULL; p++) {
      filelist[zcount] = p[0];
      lcount++;
      zcount++;
    }
    filelist[zcount] = NULL;
    printf("List of files :\n");
    CALLOC(1, sizeof(AFFY_CHIPSET));
    printf("files :\n");
    for (p = filelist, lcount = 0; *p != NULL; p++) {
      printf(" %s\n", p[0]);
      lcount++;
    }
  } else {
    /* Allocate storage for as many chips as we expect */
		filelist=argv+optind;
  }
 
  /* More efficient approach: do them one at a time */
  c=rma(filelist,&flags);
  
	/* Finally write out the entire expression array */
  affy_write_expressions(c,output_file);
  
  
  return 0;
}
