
/**************************************************************************
 *
 * Filename: affy_mas5.h
 *
 * Purpose: MAS5 declarations.
 *
 * Creation date:
 *
 * Author: Steven Eschrich
 *
 *
 * Update History
 * --------------
 * 4/8/05: Imported from old libaffy (AMH)
 *
 **************************************************************************/

#ifndef _AFFY_MAS5_H_
#define _AFFY_MAS5_H_

#include <affy.h>
#include <math.h>
#include <utils.h>

/** 
 * Structure to hold all relevant information
 * about running an instance of MAS5.0. The
 * defaults for these fields are in parenthesis, but
 * the ultimate authority for these values is defined 
 * within the file mas5/mas5_set_defaults.c.
 */
typedef struct mas5_flag_struct
{
	/** (.) Default location to look for CDF file */
  char *cdf_directory;

  /** (True) Run MAS5.0 background correction */
  Boolean use_background_correction;

  /** (False) Use a mean normalization of all probes prior to processing */
  Boolean use_mean_normalization;
  /** (500) Mean normalize all probes to target mean value */
  double mean_normalization_target_mean;

	/** (True) Scale probesets to a constant value, determined as scale_target */
  Boolean use_probeset_scaling;
	/** (500) If probeset scaling is used, this should be the target trimmed mean */
  double scale_target;
  /** (0.02) Percentage below which to remove from trimmed mean */
  double trimmed_mean_low;
  /** (0.98) Percentage above which to remove from trimmed mean */
  double trimmed_mean_high;
  
  /** (16) The number of rectangular zones on the chip */
  int K; 
	/** (100) MAS5.0 smooth parameter */
  int smooth;
	/** (0.5) MAS5.0 noiseFrac parameter */
  double NoiseFrac;
  /** (2.0e-20) MAS5.0 delta parameter */
  double delta;
  /** (0.03) MAS5.0 Contrast tau parameter */
  double contrast_tau;
  /** (10) MAS5.0 Scale tau parameter */
  double scale_tau;

} MAS5_FLAGS;

#ifdef __cplusplus
extern "C"
{
#endif

  MAS5_FLAGS *mas5_get_defaults();
  void mas5_set_defaults(MAS5_FLAGS *f);
  AFFY_CHIPSET *mas5(char **filelist, MAS5_FLAGS *f);

  int mas5_background_correction(AFFY_CHIPSET *c, MAS5_FLAGS *f);
  int mas5_signal(AFFY_CHIPSET *c, MAS5_FLAGS *f);
  int mas5_scale(AFFY_CHIPSET *c, MAS5_FLAGS *f);

#ifdef __cplusplus
};
#endif

#endif
