
/**************************************************************************
 *
 * Filename: affy_rma.h
 *
 * Purpose: RMA library declarations.
 *
 * Creation date:
 *
 * Author: Steven Eschrich
 *
 *
 * Update History
 * --------------
 * 4/8/05: Imported from old libaffy (AMH)
 *
 **************************************************************************/


#ifndef _AFFY_RMA_H_
#define _AFFY_RMA_H_

#include <affy.h>
#include <math.h>
#include <utils.h>

/** 
 * Structure to hold all relevant information
 * about running an instance of RMA. The defaults 
 * for these fields are in parenthesis, but the
 * ultimate authority for these values is defined
 * within the file rma/rma_set_defaults.c.
 */
typedef struct rma_flag_struct
{
  /** (.) Default location to look for CDF file */
  char *cdf_directory;

	/** (True) Run background correction */
  Boolean use_background_correction;

  /** (True) Run normalization step */
  Boolean use_normalization;

  /** (False) Use mean normalization instead of quantile normalization */
  Boolean use_mean_normalization;

  /** (500) If using mean normalization, set mean equal to target */
  double mean_normalization_target_value;

  /** (False) Dump probe affinities to a file */
  Boolean dump_probe_affinities;

  /** ("affinities.txt") Optional affinity filename */
  char *affinities_filename;

  /** DS: filename to save intermediate results */
  char *intermediate_file;

  /** DS: filelist of intermediate images */
  char *list_intermediate;

  /** DS: output file*/
  char *output_filename;

  /** DS: list of probesets */
  char *probe_list;

} RMA_FLAGS;

#ifdef __cplusplus
extern "C"
{
#endif

  void rma_set_defaults(RMA_FLAGS *f);
  RMA_FLAGS *rma_get_defaults();

/** 
 * General entrypoint for a typical RMA. A null pointer can be
 * passed for the flags argument (defaults are selected).
 */
 AFFY_CHIPSET *rma(char **filelist, RMA_FLAGS *f);
 AFFY_CHIPSET *combine_intermediate(char *filelist, RMA_FLAGS *f);

	/** Individual functions used in rma */
  void rma_background_correct(AFFY_CHIPSET *c, int chipnum);
  void quantile_normalization_chip(AFFY_CHIPSET *c, int chipnum,
                                   double *mean);
  void quantile_normalization_chipset(AFFY_CHIPSET *c, double *mean);
  void rma_signal(AFFY_CHIPSET *c, RMA_FLAGS *f);
  void median_polish(double **z, int numprobes, int numchips,
                     double *results, double *affinities);

#ifdef __cplusplus
};
#endif

#endif
