
/**************************************************************************
 *
 * Filename: cdf_file_util.c
 * 
 * Purpose: CDF file utility routines.
 *
 * Creation date: 27 May, 2005
 *
 * Author: Andrew M. Hoerter
 *
 *
 * Update History
 * --------------
 * 5/27/05: Creation (AMH)
 *
 **************************************************************************/

#include <affy.h>

/* 
 * affy_free_cdf_file(): Free a CDF file structure and any associated
 * storage.
 *
 * Inputs: *cdf contains an initialized CDF file structure.
 * Outputs: None.
 * Side effects: Associated memory is freed.
 */
void affy_free_cdf_file(AFFY_CDFFILE *cdf)
{
  assert(cdf != NULL);

  /* Free each dynamically allocated member. */
  if (cdf->array_type != NULL)
    free(cdf->array_type);

  if ((cdf->cell_type != NULL) && (cdf->xy_ref != NULL))
  {
    int i;

    for (i = 0; i < cdf->numrows; i++)
    {
      free(cdf->cell_type[i]);
      free(cdf->xy_ref[i]);
    }

    free(cdf->cell_type);
    free(cdf->xy_ref);
  }

  if (cdf->probeset != NULL)
    free(cdf->probeset);

  if (cdf->probe != NULL)
    free(cdf->probe);

  /* And now the structure itself. */
  free(cdf);
}
