
/**************************************************************************
 *
 * Filename: cel_file_util.c
 *
 * Purpose: Utility functions for CEL files.
 *
 * Creation date:
 *
 * Author: Steven Eschrich
 *
 *
 * Update History
 * --------------
 * 4/8/05: Imported/fixed up from old libaffy (AMH)
 * 4/19/05: Now we use AFFY_CELL's, plus a utility function (AMH)
 *
 **************************************************************************/

#include <affy.h>
#include <utils.h>

/* It's our mess, it's only fair... */
void affy_free_cel_file(AFFY_CELFILE *cf)
{

  if (cf == NULL)
    return;
  if (cf->filename != NULL)
    free(cf->filename);

  if (cf->data != NULL)
  {
    free(cf->data[0]);
    free(cf->data);
  }
  if (cf->mask != NULL)
  {
    free(cf->mask[0]);
    free(cf->mask);
  }
  if (cf->outlier != NULL)
  {
    free(cf->outlier[0]);
    free(cf->outlier);
  }

  free(cf);
  return;
}

/* 
 * affy_matrix_from_cel(): Extract cell value matrix from an AFFY_CELFILE
 *
 * Inputs: A pointer to a valid AFFY_CELFILE structure.
 *
 * Outputs: A pointer to the 2D array of values on success, NULL on failure
 *
 * Side effects: Memory is allocated which must be freed by the caller.
 *
 * Comments: For various processing it is desireable to have quick,
 *           direct access to cell values which are ordinarily hidden
 *           within AFFY_CELL's.  Thus this handy utility function to
 *           pull out those values and place them in a block of doubles.
 */

double **affy_matrix_from_cel(AFFY_CELFILE *cf)
{
  int x, y;
  double **matrix;

  assert(cf != NULL);

  /* Allocate the matrix. */
  if ((matrix = create_matrix(cf->numrows, cf->numcols)) == NULL)
    return (NULL);

  /* 
   * Copy the values over. XXX -- use pointers to data within AFFY_CELL
   * instead?
   */
  for (y = 0; y < cf->numrows; y++)
    for (x = 0; x < cf->numcols; x++)
      matrix[y][x] = cf->data[y][x].value;

  return (matrix);
}
