
/**************************************************************************
 *
 * Filename: free_chip.c
 * 
 * Purpose: Free routines for AFFY_CHIPs and AFFY_CHIPSETs.
 *
 * Creation date: 27 May, 2005
 *
 * Author: Andrew M. Hoerter
 *
 *
 * Update History
 * --------------
 * 5/27/05: Creation (AMH)
 *
 **************************************************************************/

#include <affy.h>

/* 
 * affy_free_chip():
 *
 * Inputs: *ch contains an initialized AFFY_CHIP structure.
 * Outputs: None.
 * Side effects: Associated memory is freed.
 */
void affy_free_chip(AFFY_CHIP *ch)
{
  assert(ch != NULL);

  /* Free each dynamically allocated member. */
  if (ch->cdf != NULL)
    affy_free_cdf_file(ch->cdf);

  if (ch->cel != NULL)
    affy_free_cel_file(ch->cel);
  
  if (ch->dat != NULL)
    affy_free_dat_file(ch->dat);

  if (ch->filename != NULL)
    free(ch->filename);

  if (ch->probe_set != NULL)
    free(ch->probe_set);

  /* And now the structure itself. */
  free(ch);
}

/* 
 * affy_free_chipset():
 *
 * Inputs: *ch contains an initialized AFFY_CHIPSET structure.
 * Outputs: None.
 * Side effects: Associated memory is freed.
 */
void affy_free_chipset(AFFY_CHIPSET *cs)
{
  assert(cs != NULL);

  /* Free each dynamically allocated member. */
  if (cs->cdf != NULL)
    affy_free_cdf_file(cs->cdf);

  if (cs->chip != NULL)
  {
    int i;

    for (i = 0; i < cs->numchips; i++)
    {
      assert(cs->chip[i] != NULL);

      affy_free_chip(cs->chip[i]);
    }

    free(cs->chip);
  }

  /* And now the structure itself. */
  free(cs);
}
