
/**************************************************************************
 *
 * Filename: get_cdf_name.c
 *
 * Purpose: Extract CDF filename from DatHeader in CEL file.
 *
 * Creation date: 
 *
 * Author: Steven Eschrich
 *
 *
 * Update History
 * --------------
 * 4/8/05: Imported/repaired from old libaffy (AMH)
 *
 **************************************************************************/

#include <affy.h>
#include <utils.h>

/**
 * Given a string which is the DatHeader line of a CEL file, figure
 * out the corresponding CDF file.
 */

char *affy_get_cdf_name(char *buf)
{
  char *p;

  /* Get the cdf name */
  p = strstr(buf, ".1sq");
  if (p == NULL)
    die(("Cannot find a cdf name from: %s\n", buf));
  *p = 0;
  while (p > buf && *(p - 1) != ' ' && *(p - 1) != 024)
    p--;

  return strdup(p);
}

/**
 * Given a cel file name, open it and find the corresponding
 * cdf name.
 */
#define HDR_OFFSET 20
char *affy_get_cdf_name_from_cel(char *filename)
{
  FILE *fp;
  affy_int32 magic;
  char buf[MAXBUF];

  /* Figure out if file is txt (V3) or binary (V4) */
  if (endsWith(filename, ".gz")) {
    info(("Opening compressed file...\n"));
    if ((fp = z_open(filename)) == NULL)
      die(("Couldn't read CEL file %s", filename));
  }
  else {
    fp = FOPEN(filename, "r");
  }

  if (affy_read32(fp, (void *)&magic) != 0)
    die(("Couldn't read CEL file %s", filename));

  if (magic == AFFY_CEL_BINARYFILE_MAGIC)
  {
    affy_int32 hdrlen;
    int readin = 0, i = 0;

#if 0
    fseek(fp, HDR_OFFSET, SEEK_SET);
#else
    for (i =0; i < 4; i++) {
      affy_read32(fp, (void *)&hdrlen);
    }
#endif

    affy_read32(fp, (void *)&hdrlen);
    while (fgets(buf, MAXBUF, fp) != NULL && readin < hdrlen)
    {
      if (!strncmp(buf, "DatHeader=", 10))
        break;
      readin = ftell(fp) - HDR_OFFSET + 4;
    }
  }
  else if ( (magic&0xff) == AFFY_CEL_AGCC_MAGIC ) {
    affy_agcc_get_cdf_buf(fp, buf, MAXBUF);
  }
  else
  {
    /* Find the dat header line */
    while (fgets(buf, MAXBUF, fp) != NULL)
      if (!strncmp(buf, "DatHeader=", 10))
        break;
  }
  fclose(fp);
  return affy_get_cdf_name(buf);
}

#ifdef UNIT_TEST
int main(int argc, char **argv)
{
  printf("CDF name is %s\n", affy_get_cdf_name_from_cel(argv[1]));
}
#endif
