
/**************************************************************************
 *
 * Filename: load_cdf_file.c
 *
 * Purpose: Parse a CDF file and initialize an accompanying structure.
 *
 * Creation date: 
 *
 * Author: Steven Eschrich
 *
 *
 * Update History
 * --------------
 * 4/8/05: Imported/repaired from old libaffy (AMH)
 *
 **************************************************************************/

#include <affy.h>
#include <utils.h>

/* XXX -- remove access() calls and handle in some platform-independent way */
#include <unistd.h>

void affy_load_text_cdf_file(FILE *fp, AFFY_CDFFILE *cdf, int cdf_pb);
void affy_load_binary_cdf_file(FILE *fp, AFFY_CDFFILE *cdf, int cdf_pb);
/*
  The CDF file contains the description of the microarray chip in terms
  of probes and probe sets. This information needs to be loaded into
  memory so that calculations at the probe level can be matched
  to (X,Y) coordinates in the CEL file.
  
  The parameter is a chip_type as defined by the CEL file. This should
  correspond to a cdf file somewhere.
*/
AFFY_CDFFILE *affy_load_cdf_file(char *chip_type, char *dir)
{
  AFFY_CDFFILE *cdf;
  FILE *fp;
  char cdf_filename[MAXBUF];
  int cdf_pb;

  /* Try to find cdf file. */
  if (dir != NULL)
  {
    /* Check if dir is actually a cdf file */
    if (endsWith(dir, ".cdf") && (access(dir, R_OK) == 0))
    {
      sprintf(cdf_filename, dir);
      goto FOUND;
    }

    /* Otherwise, treat it as a directory */
    sprintf(cdf_filename, "%s/%s.CDF", dir, chip_type);
    if (access(cdf_filename, R_OK) == 0)
      goto FOUND;
    sprintf(cdf_filename, "%s/%s.cdf", dir, chip_type);
    if (access(cdf_filename, R_OK) == 0)
      goto FOUND;
  }
  /* Even if not asked, check the current directory */
  sprintf(cdf_filename, "%s.CDF", chip_type);
  if (access(cdf_filename, R_OK) == 0)
    goto FOUND;
  sprintf(cdf_filename, "%s.cdf", chip_type);
  if (access(cdf_filename, R_OK) == 0)
    goto FOUND;

  /* If we got here, can't find it and die */
  die(("Cannot find cdf file anywhere!\n"));
FOUND:

  /* Open file */
  fp = FOPEN(cdf_filename, "r");
  setvbuf(fp, NULL, _IOFBF, MAXBUF);
  cdf = CALLOC(1, sizeof(AFFY_CDFFILE));

  info(("Loading %s CDF file:\n", cdf_filename));
  cdf_pb = pb_create(100, "Loading CDF File");

	/* Do a detection of the type, then call the appropriate code. */
	if ( affy_readint(fp) == AFFY_CDF_BINARYFILE_MAGIC ) {
		/* Reopen cdf file in binary mode - for PC's */
		fclose(fp);
	  if ((fp = fopen(cdf_filename, "rb")) == NULL)
  	  return NULL;

		/* Load the file */
		affy_load_binary_cdf_file(fp,cdf,cdf_pb);
	} else {
		rewind(fp);
		affy_load_text_cdf_file(fp,cdf,cdf_pb);
	}
	fclose(fp);
	pb_finish(cdf_pb);

 	return cdf;
}
