
/**************************************************************************
 *
 * Filename: load_cel_file.c
 *
 * Purpose: Parse a CEL file and initialize an accompanying structure.
 *
 * Creation date: 
 *
 * Author: Steven Eschrich
 *
 *
 * Update History
 * --------------
 * 4/8/05: Imported/repaired from old libaffy (AMH)
 * 4/19/05: Now we use AFFY_CELL and collect all cell data (AMH)
 *
 **************************************************************************/

#include <affy.h>
#include <utils.h>

void affy_load_text_cel_file(FILE *fp, AFFY_CELFILE *cf, int cf_pb);



AFFY_CELFILE *affy_load_cel_file(char *filename)
{
  char *str;
  FILE *fp;
  AFFY_CELFILE *cf = NULL;
	int pb;
	int magic;
	
  if (!filename)
    return NULL;

  /* Open file */
  if (endsWith(filename, ".gz")) {
    info(("Opening compressed file...\n"));
    if ((fp = z_open(filename)) == NULL)
      return NULL;
  }
  else {
    if ((fp = fopen(filename, "r")) == NULL)
      return NULL;
  }
  info(("Loading %s CEL file:\n", filename));
  pb = pb_create(100, "Loading CEL File");

  /* Start allocating parts of ds */
  cf = CALLOC(1, sizeof(AFFY_CELFILE));
  cf->filename = strdup(filename);

  magic = affy_readint(fp);
	/* Check and call appropriate version */
	if ( magic == AFFY_CEL_BINARYFILE_MAGIC ) {
		/* Reopen cel file in binary mode - for PC's */
		fclose(fp);
		if (endsWith(filename, ".gz")) {
		  info(("Opening compressed file...\n"));
		  if ((fp = z_open(filename)) == NULL)
		    return NULL;
		}
		else {
		  if ((fp = fopen(filename, "rb")) == NULL)
		    return NULL;
		}
		/* Load the file */
		affy_load_binary_cel_file(fp, cf, pb);
	} else if ( (magic&0xff) == AFFY_CEL_AGCC_MAGIC ) {
		/* Reopen cel file in binary mode - for PC's */
		fclose(fp);
		if (endsWith(filename, ".gz")) {
		  info(("Opening compressed file...\n"));
		  if ((fp = z_open(filename)) == NULL)
		    return NULL;
		}
		else {
		  if ((fp = fopen(filename, "rb")) == NULL)
		    return NULL;
		}
		/* Load the file */
		affy_load_agcc_cel_file(fp, cf, pb);
	} else {
		/* rewind(fp); */
		fclose(fp);
		if (endsWith(filename, ".gz")) {
		  info(("Opening compressed file...\n"));
		  if ((fp = z_open(filename)) == NULL)
		    return NULL;
		}
		else {
		  if ((fp = fopen(filename, "r")) == NULL)
		    return NULL;
		}
		affy_load_text_cel_file(fp, cf, pb);
	}
	

	
	fclose(fp);
  pb_finish(pb);
	
	return cf;
}
