
/**************************************************************************
 *
 * Filename: load_data..c
 *
 * Purpose: Load a collection of Affymetrix data files all at once.
 *
 * Creation date: 
 *
 * Author: Steven Eschrich
 *
 *
 * Update History
 * --------------
 * 4/8/05: Imported/repaired from old libaffy (AMH)
 *
 **************************************************************************/

#include <affy.h>
#include <utils.h>

/*
  affy_load_data()

  Load the affymetrix data listed in directory dir and specified
  by files. Either of dir or files can be NULL (but not both!).

  The CEL files names by files are opened, as is the corresponding
  CDF file. The resulting CHIPSET is returned as a dynamically
  allocated object.
*/

AFFY_CHIPSET *affy_load_data(char *dir, char **files)
{
  AFFY_CHIPSET *c;
  char **p;
  int max_chips;
  int i, j;
  char **filelist = files;

  char *chip_type;

  if (dir == NULL && files == NULL)
    return NULL;

  c = CALLOC(1, sizeof(AFFY_CHIPSET));

  /* If files is NULL, open all CEL files in the current directory */
  if (files[0] == NULL)
  {
    filelist = affy_list_files(dir, ".cel");
  }
  else
  {
    /* Allocate storage for as many chips as we expect */
    filelist = files;
  }

  for (p = filelist, max_chips = 0; p != NULL; p++)
    max_chips++;
  c->numchips = max_chips;
  c->chip = CALLOC(max_chips, sizeof(AFFY_CHIP *));

  /* Now load the cdf file (figure out chip type) */
  chip_type = affy_get_cdf_name_from_cel(filelist[0]);
  c->cdf = affy_load_cdf_file(chip_type, NULL);

  /* Load each chip */
  for (i = 0, j = 0; i < max_chips; i++)
  {
    c->chip[j] = affy_load_chip(filelist[i]);
    if (c->chip[j] != NULL)
      j++;
  }
  c->numchips = j;

  /* A kludge: redistribute the cdf ptr out to cel file structures */
  for (i = 0; i < c->numchips; i++)
    c->chip[i]->cdf = c->cdf;
  c->numrows = c->cdf->numrows;
  c->numcols = c->cdf->numcols;

  if (files == NULL)
    free(filelist);

  return c;
}
