
/**************************************************************************
 *
 * Filename: text_io.c
 *
 * Purpose: Routines for dealing with plain text I/O.
 *
 * Creation date: 
 *
 * Author: Steven Eschrich
 *
 *
 * Update History
 * --------------
 * 4/8/05: Imported/repaired from old libaffy (AMH)
 *
 **************************************************************************/

#include <affy.h>
#include <utils.h>

/*==================================================================
  Simple line reader with undo
*===================================================================*/

static Boolean skip_read = False;
static char buf[MAXBUF];

/* Read a single line from the data file, trim it, and return it */
char *affy_get_next_line(FILE * fp)
{
  char *p = NULL;

  while (1)
  {

    /* A special flag allows us to unget a line */
    if (skip_read)
      skip_read = False;
    else if (fgets(buf, MAXBUF, fp) == NULL)
      return NULL;

    /* Trim the whitespace */
    p = trim(buf);

    /* If not empty, return it */
    if (strlen(p) > 0)
      return p;
  }

  return p;
}

void affy_unget_next_line(FILE * fp)
{
  skip_read = True;
  return;
}

void affy_reset_next_line(FILE * fp)
{
  skip_read = False;
  return;
}

void affy_skip_to_next_header(FILE * fp)
{
  char *s;

  while ((s = affy_get_next_line(fp)) != NULL)
    if (*s == '[')
    {
      affy_unget_next_line(fp);
      return;
    }

  return;
}
