
/**************************************************************************
 *
 * Filename: write_expressions.c
 *
 * Purpose: Write expression data from a set of chips to a given file
 *
 * Creation date: 
 *
 * Author: Steven Eschrich
 *
 *
 * Update History
 * --------------
 * 4/8/05: Imported/repaired from old libaffy (AMH)
 *
 **************************************************************************/

#include <affy.h>
#include <utils.h>

void affy_write_expressions(AFFY_CHIPSET *c, char *filename)
{
  int n, i;
  FILE *fp;

  fp = FOPEN(filename, "w");

  /* Print the header first */
  fprintf(fp, "%s\t", filename);
  for (i = 0; i < c->numchips; i++) {
		char *p=c->chip[i]->filename;
		char *q, *e;
		
		/* Strip out leading path information */
		q=strrchr(p,'/');
		/* Either we found it (increment) or not */
		if ( q != NULL )
			q++;
		else {
			q=strrchr(p,'\\');
			/* Either we found it (again) or not */
			if ( q == NULL )
				q=p;
			else
				q++;
		}
		/* q is now the string without leading / or \ */
		q=strdup(q);
		
		/* Strip out extension as well */
		e=strrchr(q,'.');
		if (e != NULL && strncmp(e, ".gz", 3) == 0) {
		  *e=0;
		  e=strrchr(q,'.');
		}
		if ( e != NULL ) {
			*e=0;
		}
    fprintf(fp, "%s%c", q,
            i < c->numchips - 1 ? '\t' : '\n');
  }
  
  /* Now the data */
  for (i = 0; i < c->cdf->numprobesets; i++)
  {
    fprintf(fp, "%s", c->cdf->probeset[i].name);
    for (n = 0; n < c->numchips; n++)
      fprintf(fp, "\t%f", c->chip[n]->probe_set[i]);
    fprintf(fp, "\n");
  }
  fclose(fp);
  return;
}
