/**************************************************************************
 *
 * Filename: write_pixel_region.c
 *
 * Purpose: Write pixels from a DAT to a given file. See function at
 * bottom of file for attempt at generic interface.
 *
 * Creation date: 7/20/05
 *
 * Author: Steven Eschrich
 *
 *
 * Update History
 * --------------
 *
 **************************************************************************/
#include "affy.h"


#ifdef TIFF_SUPPORT

#include <tiffio.h>

void affy_pixregion2tiff(AFFY_PIXREGION *p, char *filename)
{
	int i,j;
	TIFF *fp;
	uint16 *x;
    
	fp=TIFFOpen(filename,"w");

	
	if ( fp ) {
		x=calloc(p->numcols,sizeof(uint16));
		TIFFSetField(fp,TIFFTAG_BITSPERSAMPLE,16);
		TIFFSetField(fp,TIFFTAG_IMAGELENGTH,p->numrows);
		TIFFSetField(fp,TIFFTAG_IMAGEWIDTH,p->numcols);
		TIFFSetField(fp,TIFFTAG_PLANARCONFIG,PLANARCONFIG_CONTIG);
		for (i=0; i < p->numrows; i++) {
			for (j=0; j < p->numcols;j++) {
				x[j]=p->data[i][j];
			}
			TIFFWriteScanline(fp,x, i,1);
		}
		TIFFClose(fp);
		free(x);	
	}
	
}
#else
	void affy_pixregion2tiff(AFFY_PIXREGION *p, char *filename)
	{
		fprintf(stderr,"TIFF support not enabled in libaffy, please recompile!\n");
		exit(-1);
	}
#endif

void affy_pixregion2text(AFFY_PIXREGION *p, char *filename)
{
	FILE *fp;
	int i,j;
	
	fp=fopen(filename,"w");
	
	if ( fp ) {
		for (i=0; i < p->numrows; i++) {
			for (j=0; j < p->numcols; j++) {
				fprintf(fp,"%d",p->data[i][j]);
				if ( j < p->numcols-1 ) fprintf(fp,"\t");
			}
			fprintf(fp,"\n");
		}
		fclose(fp);
	}
}


/*
 * This function could be used as a default preference of what
 * image writer to use at compile time. Could be more sophisticated.
 */
void affy_write_pixel_region(AFFY_PIXREGION *pr,char *filename)
{
	#ifdef TIFF_SUPPORT
		affy_pixregion2tiff(pr,filename);
	#else
		affy_pixregion2text(pr,filename);
	#endif
		
}
