
/**************************************************************************
 *
 * Filename: mas5.c
 *
 * Purpose: 
 *
 * Creation date: 
 *
 * Author: Steven Eschrich
 *
 *
 * Update History
 * --------------
 * 4/14/05: Imported/repaired from old libaffy (AMH)
 * 4/19/05: Use affy_free_cel_file() instead of manual code (AMH)
 *
 **************************************************************************/

#include <affy_mas5.h>

AFFY_CHIPSET *mas5(char **filelist, MAS5_FLAGS *f)
{
  AFFY_CHIPSET *c;
  AFFY_CHIP **all_chips;
  int i, j;
  char *chip_type;
  char **p;
  int max_chips = 0;

  /* In case flags not used, create default entry */
  if (f == NULL)
    f = mas5_get_defaults();

  /* Count up chips */
  for (p = filelist, max_chips = 0; *p != NULL; p++)
    max_chips++;

  /* Create structure */
  c = CALLOC(1, sizeof(AFFY_CHIPSET));
  c->numchips = 1;
  c->chip = CALLOC(1, sizeof(AFFY_CHIP *));
  all_chips = CALLOC(max_chips, sizeof(AFFY_CHIP *));

  /* Now load the cdf file (figure out chip type) */
  chip_type = affy_get_cdf_name_from_cel(filelist[0]);
  c->cdf = affy_load_cdf_file(chip_type, f->cdf_directory);
  c->numrows = c->cdf->numrows;
  c->numcols = c->cdf->numcols;

  /* Load each chip */
  for (i = 0, j = 0; i < max_chips; i++)
  {
    /* Load chip */
    c->chip[0] = affy_load_chip(filelist[i]);
    if (c->chip[0] == NULL)
      continue;
    /* Kludge: distribute cdf ptr */
    c->chip[0]->cdf = c->cdf;

    /* Process chip */
    if (f->use_mean_normalization)
      affy_mean_normalization(c, f->mean_normalization_target_mean);
    if (f->use_background_correction)
      mas5_background_correction(c, f);
    mas5_signal(c, f);
    if (f->use_probeset_scaling)
      mas5_scale(c, f);

    /* Accumulate expressions */
    all_chips[j] = c->chip[0];

    /* Free chip space */
    affy_free_cel_file(all_chips[j]->cel);
    all_chips[j]->cel = NULL;

    j++;
  }

  /* Finally return the entire expression array */
  c->chip = all_chips;
  c->numchips = max_chips;
  return c;

}
