
/**************************************************************************
 *
 * Filename: mas5_scale.c
 *
 * Purpose: 
 *
 * Creation date: 
 *
 * Author: Steven Eschrich
 *
 *
 * Update History
 * --------------
 * 4/14/05: Imported/repaired from old libaffy (AMH)
 *
 **************************************************************************/

#include <affy_mas5.h>

static double trimmed_mean(double *values, int n, double lo, double hi);

int mas5_scale(AFFY_CHIPSET *c, MAS5_FLAGS *f)
{
  int i, n;
  int progress, incr;
  int num_probesets;
  double *signal;
  double sf = 0;

  if (c == 0 || c->numchips == 0 || c->cdf == 0)
    return -1;
  if (f == NULL)
    f = mas5_get_defaults();

  num_probesets = c->cdf->numprobesets;
  signal = CALLOC(num_probesets, sizeof(double));

  for (n = 0; n < c->numchips; n++)
  {
    progress = incr = c->cdf->numprobesets / 10;
    info(("  Scaling probeset values to %5.1f: ", f->scale_target));

    /* Load signal and get scaling factor */
    for (i = 0; i < num_probesets; i++)
    {
      signal[i] = c->chip[n]->probe_set[i];
    }
    sf =
      f->scale_target / trimmed_mean(signal, num_probesets,
                                     f->trimmed_mean_low,
                                     f->trimmed_mean_high);
    info((" sf=%f ", sf));
    /* Store adjusted value */
    for (i = 0; i < num_probesets; i++)
    {
      c->chip[n]->probe_set[i] *= sf;
    }
    info(("done.\n"));

  }
  free(signal);

  return 0;
}

/* 
 * Return the trimmed mean of a list of numbers, trimmed by the
 * top lo and hi percentages.
 */
static double trimmed_mean(double *values, int n, double lo, double hi)
{
  double sum = 0;
  int min_value;
  int max_value;
  int i;

  /* Calculate number to start/stop at in sorted list */
  min_value = (int)(n * lo);
  max_value = (int)(n * hi)-1;

  /* Sort values */
  qsort(values, n, sizeof(double), dcompare);

  /* Accumulate sums */
  for (i = min_value; i < max_value; i++)
  {
    sum += values[i];
  }

  /* Divide by num signals */
  sum = sum / (max_value - min_value);

  return sum;
}
