
/**************************************************************************
 *
 * Filename: combine_intermediate.c
 *
 * Purpose: Combine intermediate results to generate an expression file
 *
 * Creation date: 5/9/07
 *
 * Author: Debashis Sahoo
 *
 *
 * Update History
 * --------------
 * 5/9/07: Created
 *
 * $Id: combine_intermediate.c,v 1.2 2007/05/10 05:00:22 sahoo Exp $
 *
 **************************************************************************/


#include <affy_rma.h>

extern int rma_pb;

/*************  Hashtable ********************/

static unsigned long sdbm(unsigned char *str) {
  unsigned long hash = 0;
  int c;

  while (c = *str++)
    hash = c + (hash << 6) + (hash << 16) - hash;

  return hash;
}

static unsigned int hashfromkey(void *ky) {
  unsigned long h = sdbm( (unsigned char *) ky);
  return (unsigned int) h;
}

static int equalkeys(void *ky1, void *ky2) {
  return strcmp( (char*) ky1, (char*)ky2 ) == 0;
}

/*************  END Hashtable ********************/

int iswhite(char c)
{
  return (c <= 0x20 || c >= 0x7F);
}

void my_trim(char *str) {
  int i = strlen(str)-1;
  for (; i >= 0; i--) {
    if (iswhite(str[i]))
      str[i] = '\0';
    else 
      break;
  }
}

/* Copied from affy_write_expressions() */
void print_header_data(FILE* fp, AFFY_CHIPSET * c, RMA_FLAGS *f) {
  int i = 0;

  /* Print the header first */
  fprintf(fp, "%s\t", f->output_filename);
  for (i = 0; i < c->numchips; i++) {
		char *p=c->chip[i]->filename;
		char *q, *e;
		
		/* Strip out leading path information */
		q=strrchr(p,'/');
		/* Either we found it (increment) or not */
		if ( q != NULL )
			q++;
		else {
			q=strrchr(p,'\\');
			/* Either we found it (again) or not */
			if ( q == NULL )
				q=p;
			else
				q++;
		}
		/* q is now the string without leading / or \ */
		q=strdup(q);
		
		/* Strip out extension as well */
		e=strrchr(q,'.');
		if (e != NULL && strncmp(e, ".gz", 3) == 0) {
		  *e=0;
		  e=strrchr(q,'.');
		}
		if ( e != NULL ) {
			*e=0;
		}
    fprintf(fp, "%s%c", q,
            i < c->numchips - 1 ? '\t' : '\n');
  }
  
}

void print_probe_info(AFFY_CHIPSET * c, RMA_FLAGS *f, int numfiles,
    AFFY_PSLOAD *psload)
{
  FILE *fp;
  FILE *fout;
  char line[MAXBUF];
  int numps = 0;
  char **listps = NULL;
  int ps;
  AFFY_CDFFILE *cdf = c->cdf;
  int numchips = c->numchips;
  int i = 0, j = 0, k = 0;
  int curChip = 0;
  int progress, incr;
  struct hashtable * h;
  char * allprobe = "all";
  double *results;
  double *affinities;
  int affinity_size = 0;
  
  h = create_hashtable(16, hashfromkey, equalkeys);

  info(("reading probe list: %s\n", f->probe_list));
  fp = FOPEN(f->probe_list, "r");
  setvbuf(fp, NULL, _IOFBF, MAXBUF);
  while (fgets(line, MAXBUF, fp) != NULL) {
    char * k = strtok(line, "\t");
    char * v = strtok(NULL, "\t");
    char * val = allprobe;
    if (v != NULL) {
      val = v;
    }
    if (numps % REALLOC_INCR == 0)
      listps = REALLOC(listps, (numps + REALLOC_INCR) * sizeof(char *));
    my_trim(k);
    listps[numps] = CALLOC(strlen(k) + 1, sizeof(char));
    sprintf(listps[numps], "%s", k);
    if (!hashtable_insert(h,strdup(k),strdup(val))) exit(-1); /*oom*/
    numps++;
  }
  fclose(fp);
  info(("Done\n"));

  fout = FOPEN(f->output_filename, "w");
  print_header_data(fout, c, f);

  results = CALLOC(numchips, sizeof(double));
  progress = incr = cdf->numprobesets / 100;
  for (ps = 0; ps < cdf->numprobesets; ps++)
  {
    AFFY_PROBESET *p = &(cdf->probeset[ps]);
    int numprobes = p->numprobes;
    double **z;
    char * val = allprobe;

    if (ps == progress)
    {
      pb_update(rma_pb, progress / incr, "");
      progress += incr;
    }

    if (hashtable_search(h,p->name) == NULL) {
      continue;
    }
    val = (char*) hashtable_search(h,p->name);

    /* Allocate storage for z matrix (for polishing) */
    z = create_matrix(numprobes, numchips);

    /* load matrix from disk */
    curChip = 0;
    for (i = 0; i < numfiles; i++) {
      double **z1 = create_matrix(numprobes, psload[i].numchips);
      load_matrix_from_disk(&psload[i], ps, z1);

      /* Load logged data for this probe set */
      for (k = 0; k < psload[i].numchips; k++)
      {
	/* Load probes from pm array or actual cel data */
	for (j = 0; j < numprobes; j++)
	{
	  z[j][curChip] = z1[j][k];
	}
	curChip++;
      }
      free_matrix(z1);
    }
    if (strcmp(val, allprobe) == 0) {
      for (j = 0; j < numprobes; j++) {
	fprintf(fout, "%s-%d", p->name, j);
	for (i = 0; i < numchips; i++)
	  fprintf(fout, "\t%f", z[j][i]);
	fprintf(fout, "\n");
      }
    }
    else {
      char * v = strtok(val, ",");
      int num = 0;
      double **z2;
      /* Allocate storage for z matrix (for polishing) */
      z2 = create_matrix(numprobes, numchips);

      for (; v != NULL; v = strtok(NULL, ",")) {
	if (strcmp(v, "") == 0) {
	  continue;
	}
	int probe = atoi(v);
	for (i = 0; i < numchips; i++)
	  z2[num][i] = z[probe][i];
	num ++;
      }

      /* Even if affinities aren't stored, the median polish will record them */
      if (affinity_size < numprobes)
      {
	affinities = REALLOC(affinities, numprobes * sizeof(double));
	affinity_size = numprobes;
      }

      /* Do the median polish */
      median_polish(z2, num, numchips, results, affinities);
      free_matrix(z2);

      fprintf(fout, "%s", p->name);
      for (i = 0; i < numchips; i++)
	fprintf(fout, "\t%f", results[i]);
      fprintf(fout, "\n");
    }
  }
  fclose(fout);
  free(results);
  free(affinities);
  if (h != NULL) {
    hashtable_destroy(h,1 /* free vals also */);
  }
}

AFFY_CHIPSET *combine_intermediate(char *filelist, RMA_FLAGS *f)
{
  AFFY_CHIPSET * c = NULL;
  FILE *fp;
  FILE *fout;
  char line[MAXBUF];
  int numfiles = 0;
  char **listfiles = NULL;
  char **p;
  int max_chips = 0;
  char *chip_type = NULL;
  double *mean = NULL;
  double *mn = NULL;
  int numprobes, curChip;
  int i, j, k;
  int ps;
  double *results;
  double *affinities;
  AFFY_CDFFILE *cdf;
  int numchips;
  double LOG2 = log(2.0);
  int affinity_size = 0;
  int progress, incr;
  AFFY_PSLOAD *psload;

  info(("Combining intermediate images in %s\n", filelist));
  fp = FOPEN(filelist, "r");
  setvbuf(fp, NULL, _IOFBF, MAXBUF);
  while (fgets(line, MAXBUF, fp) != NULL) {
    if (numfiles % REALLOC_INCR == 0)
      listfiles = REALLOC(listfiles, (numfiles + REALLOC_INCR) * sizeof(char *));
    my_trim(line);
    listfiles[numfiles] = CALLOC(strlen(line) + 1, sizeof(char));
    sprintf(listfiles[numfiles], "%s", line);
    numfiles++;
  }
  fclose(fp);
  /* Last spot in list is null */
  if (numfiles % REALLOC_INCR == 0)
    listfiles = REALLOC(listfiles, (numfiles + 1) * sizeof(char *));
  listfiles[numfiles] = NULL;

  if (numfiles <= 0) {
    return NULL;
  }

  psload = CALLOC(numfiles, sizeof(AFFY_PSLOAD));
  for (i = 0; i < numfiles; i++) {
    /* Create structure */
    c = CALLOC(1, sizeof(AFFY_CHIPSET));

    info((" %s\n", listfiles[i]));
    init_probeset_loading(listfiles[i], c, &psload[i]);
    max_chips += psload[i].numchips;
    if (!chip_type) {
      chip_type = strdup(psload[i].chiptype);
    }
    c->chip = CALLOC(psload[i].numchips, sizeof(AFFY_CHIP *));
    /* Now load the cdf file (figure out chip type) */
    info(("Chiptype: %s\n", psload[i].chiptype));
    c->cdf = affy_load_cdf_file(psload[i].chiptype, f->cdf_directory);
    c->numrows = c->cdf->numrows;
    c->numcols = c->cdf->numcols;
    psload[i].h_probesets = get_probesets(c->cdf);
  }

  /* Create structure */
  c = CALLOC(1, sizeof(AFFY_CHIPSET));

  c->chip = CALLOC(max_chips, sizeof(AFFY_CHIP *));

  /* Now load the cdf file (figure out chip type) */
  info(("Chiptype: %s\n", chip_type));
  c->cdf = affy_load_cdf_file(chip_type, f->cdf_directory);
  cdf = c->cdf;
  c->numrows = c->cdf->numrows;
  c->numcols = c->cdf->numcols;

  numprobes = c->cdf->numprobes;

  for (ps = 0; ps < cdf->numprobesets; ps++)
  {
    AFFY_PROBESET *p = &(cdf->probeset[ps]);
    int numcprobes = p->numprobes;
    for (i = 0; i < numfiles; i++) {
      int val = (int) hashtable_search(psload[i].h_probesets,p->name);
      int ncp = psload[i].chipset->cdf->probeset[val].numprobes;
      assert(numcprobes == ncp);
    }
  }

  /* load filename */
  info(("Loading filename ...\n"));
  curChip = 0;
  for (i = 0; i < numfiles; i++) {
    for (k = 0; k < psload[i].numchips; k++) {
      c->chip[curChip] = CALLOC(1, sizeof(AFFY_CHIP));
      c->chip[curChip]->cel = NULL;
      c->chip[curChip]->filename = load_filename_from_disk(&psload[i], k);
      info((" %s\n", c->chip[curChip]->filename));
      curChip++;
    }
  }

  info(("Loading Mean ...\n"));
  
  mean = CALLOC(c->cdf->numprobes, sizeof(double));
  bzero(mean, c->cdf->numprobes * sizeof(double));
  
  mn = CALLOC(c->cdf->numprobes, sizeof(double));
  for (p = listfiles; *p != NULL; p++) {
    int np = 0;
    load_mean_from_disk(mn, &np, p[0]);
    assert(numprobes == np);
    for (i = 0; i < np; i++) {
      mean[i] += mn[i];
      if(isnan(mean[i])) {
	info(("mean[%d] is NAN\n", i));
	exit(1);
      }
    }
  }
  free(mn);

  c->numchips = max_chips;

  for (i = 0; i < c->cdf->numprobes; i++) {
    mean[i] /= c->numchips;
  }
  c->mean = mean;
  for (i = 0; i < numfiles; i++) {
    psload[i].chipset->mean = mean;
  }

  if (f->probe_list) {
    print_probe_info(c, f, numfiles, psload);
    free(psload);
    info(("done.\n"));
    free(chip_type);
    return NULL;
  }
  /* code rma_signal */
  numchips = c->numchips;
  cdf = c->cdf;
  results = CALLOC(numchips, sizeof(double));
#if 0
  /* Allocate storage */
  for (i = 0; i < numchips; i++)
  {
    c->chip[i]->probe_set = CALLOC(cdf->numprobesets, sizeof(double));
    c->chip[i]->numprobesets = cdf->numprobesets;
  }
#endif
  fout = FOPEN(f->output_filename, "w");
  print_header_data(fout, c, f);

  affinities = NULL;
  /* Median polish each probeset. The results are stored in probeset */
  pb_update(rma_pb, 1, "Calculating expressions");
  progress = incr = cdf->numprobesets / 100;
  for (ps = 0; ps < cdf->numprobesets; ps++)
  {
    AFFY_PROBESET *p = &(cdf->probeset[ps]);
    int numprobes = p->numprobes;
    double **z;

    if (ps == progress)
    {
      pb_update(rma_pb, progress / incr, "");
      progress += incr;
    }

    /* Allocate storage for z matrix (for polishing) */
    z = create_matrix(numprobes, numchips);

    /* load matrix from disk */
    curChip = 0;
    for (i = 0; i < numfiles; i++) {
      int val = (int) hashtable_search(psload[i].h_probesets,p->name);
      double **z1 = create_matrix(numprobes, psload[i].numchips);
      load_matrix_from_disk(&psload[i], val, z1);

      /* Load logged data for this probe set */
      for (k = 0; k < psload[i].numchips; k++)
      {
	/* Load probes from pm array or actual cel data */
	for (j = 0; j < numprobes; j++)
	{
	  z[j][curChip] = z1[j][k];
	}
	curChip++;
      }
      free_matrix(z1);
    }

    /* Even if affinities aren't stored, the median polish will record them */
    if (affinity_size < numprobes)
    {
      affinities = REALLOC(affinities, numprobes * sizeof(double));
      affinity_size = numprobes;
    }

    /* Do the median polish */
    median_polish(z, numprobes, numchips, results, affinities);

    free_matrix(z);

#if 0
    /* Store results */
    for (i = 0; i < numchips; i++)
    {
      c->chip[i]->probe_set[ps] = results[i];
    }
#endif

    /* write the data */
    fprintf(fout, "%s", p->name);
    for (i = 0; i < numchips; i++)
      fprintf(fout, "\t%f", results[i]);
    fprintf(fout, "\n");

    /* If desired, dump the probe affinities */
    if (f->dump_probe_affinities)
    {
      FILE *fp;
      char *af_name = f->affinities_filename;

      fp = FOPEN(af_name, "a");
      fprintf(fp, "%s", p->name);
      for (i = 0; i < numprobes; i++)
        fprintf(fp, "\t%g", affinities[i]);
      fprintf(fp, "\n");
      fclose(fp);
    }
  }
  for (i = 0; i < numfiles; i++) {
    close_probeset_loading(&psload[i]);
  }
  /* Free up results storage */
  free(results);
  free(affinities);
  free(psload);
  fclose(fout);

  info(("done.\n"));

  free(chip_type);
  return NULL;
}
