
/**************************************************************************
 *
 * Filename: max_density.c
 *
 * Purpose: 
 *
 * Creation date: 
 *
 * Author: Steven Eschrich
 *
 *
 * Update History
 * --------------
 * 4/14/05: Imported/repaired from old libaffy (AMH)
 *
 **************************************************************************/

/* This function was adapted from the Bioconductor RMA implementation */

/**************************************************************************
 **
 ** double max_density(double *z,int rows,int cols,int column, SEXP fn,SEXP rho)
 **
 ** double *z - matrix of dimension rows*cols
 ** int cols - matrix dimension
 ** int rows - matrix dimension
 ** int column - column of interest
 ** SEXP fn - R function for estimation of density
 ** SEXP rho - an R environment to work within
 **************************************************************************/

#include <utils.h>

static int density_estimate_points = 16384;

void kernel_density(double *x, int nx, double *weights, double *dy,
                    double *dx, int N);

double max_density(double *x, int n)
{

  int i, imax;
  double *dx, *dy;
  double final_result;
  double *weights;

  /* Assign standard weights */
  weights = CALLOC(n, sizeof(double));
  for (i = 0; i < n; i++)
    weights[i] = 1.0;

  /* Space for output */
  dx = CALLOC(density_estimate_points, sizeof(double));
  dy = CALLOC(density_estimate_points, sizeof(double));

  kernel_density(x, n, weights, dy, dx, density_estimate_points);

  /* Get the dx value corresponding to max dy */
  for (imax = 0, i = 1; i < density_estimate_points; i++)
    if (dy[imax] < dy[i])
      imax = i;
  final_result = dx[imax];

  free(dx);
  free(dy);
  free(weights);

  return final_result;
}
