
/**************************************************************************
 *
 * Filename: mean_normalization.c
 *
 * Purpose: Statistical routines.
 *
 * Creation date: 
 *
 * Author: Steven Eschrich
 *
 *
 * Update History
 * --------------
 * 4/14/05: Imported/repaired from old libaffy (AMH)
 * 4/19/05: AFFY_CELFILE now uses AFFY_CELL's (AMH)
 *
 **************************************************************************/

/* 
   affy_mean_normalization()

   Normalize the chips to the same constant mean intensity
*/
#include <affy.h>
#include <utils.h>

void affy_mean_normalization(AFFY_CHIPSET *d, double target_mean)
{
  int i, j, x, y;
  double mean;
  AFFY_CELFILE *cf;
  int number_of_probes;

  info(("Mean normalization..."));

  number_of_probes = d->cdf->numprobes;

  /* We start by creating a matrix of double pointers to data */
  for (i = 0; i < d->numchips; i++)
  {
    cf = d->chip[i]->cel;
    mean = 0;

    /* Calculate mean */
    for (j = 0; j < number_of_probes; j++)
    {
      x = d->cdf->probe[j]->pm.x;
      y = d->cdf->probe[j]->pm.y;
      mean += cf->data[x][y].value;
      x = d->cdf->probe[j]->mm.x;
      y = d->cdf->probe[j]->mm.y;
      mean += cf->data[x][y].value;
    }

    mean /= (2 * number_of_probes);

    /* Now shift all values by the same factor */
    for (j = 0; j < number_of_probes; j++)
    {
      x = d->cdf->probe[j]->pm.x;
      y = d->cdf->probe[j]->pm.y;
      cf->data[x][y].value *= target_mean / mean;
      x = d->cdf->probe[j]->mm.x;
      y = d->cdf->probe[j]->mm.y;
      cf->data[x][y].value *= target_mean / mean;
    }
  }

  info(("done.\n"));

  return;
}
