/**************************************************************************
 *
 * Filename: buffered_reader.h
 *
 * Purpose: Create a low level buffered reader
 *
 * Creation date: 5/14/07
 *
 * Author: Debashis Sahoo
 *
 *
 * Update History
 * --------------
 * 5/14/07: Created
 *
 * $Id:$
 *
 **************************************************************************/

#ifndef __BUFFERED_READER_H
#define __BUFFERED_READER_H

#include<stdio.h>

/*
 * Buffered reader data structure (By Debashis Sahoo)
 */
typedef struct buffered_reader_s {
  FILE *fp;
  char *buffer;
  int bufsize; /* Max size of the buffer */
  int startpos; /* Current index in the buffer */
  int endpos; /* End index in the buffer */
  long curpos; /* Current file pointer */
} buffered_reader;

/** Function prototypes. **/

#ifdef __cplusplus
extern "C"
{
#endif

  buffered_reader * init_buffered_reader(FILE *fp, int size);
  void close_buffered_reader(buffered_reader *res);
  int read_byte(buffered_reader * b);
  int fill_buffer(buffered_reader * b);
  void invalidate(buffered_reader * b);
  int read_buffered(buffered_reader * b, char *buf, int offset, int len);
  long get_filepointer(buffered_reader * b);
  int seek_buffered(buffered_reader * b, long pos);

#ifdef __cplusplus
};
#endif

#endif
