#ifndef __UTIL_LOG_H__
#define __UTIL_LOG_H__

/*********************************************************************
 * 
 * Logging utilities.
 * The method used is to define generic macros that capture some extra
 * information, etc. and then pass them off to actual implementations.
 * These implementations are part of separate libraries (libtxtlog and
 * libwxlog), so that different types of implementations are provided.
 *********************************************************************/

/* Logging utilities */
#ifdef __cplusplus
extern "C"
{
#endif
  void die_impl(const char *msg, ...);
  void debug_impl(int level, const char *msg, ...);
  void warn_impl(const char *msg, ...);
  void info_impl(const char *msg, ...);
  void status_impl(const char *msg, ...);

/* Progress bar utilities */
  int pb_create(int max, char *title, ...);
  void pb_update(int, int, char *, ...);
  void pb_finish(int);

  extern int debug_level;       /* User-defined, used by debug() */
  extern unsigned int _loc_line;
  extern const char *_loc_file;
#ifdef __cplusplus
}
#endif

#define info(args)   info_impl args
#define status(args) status_impl args

/* Take note: this stuff is very, very thread-unsafe. */
#define warn(args)  do                                                 \
                    { set_locus(__FILE__, __LINE__); warn_impl args; } \
                    while (0)

#define debug(args) do                                                   \
                    { set_locus(__FILE__, __LINE__); debug_impl args; }  \
                    while (0)

#define die(args)  do                                                 \
                   { set_locus(__FILE__, __LINE__); die_impl args; }  \
                   while (0)

#endif
